/* MoreBalance - load balancer and networking tool
 * Copyright (C) 2002 - 2006 Josef Spillner <josef@coolprojects.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* MoreBalance includes */
#include "conf.h"
#include "server.h"
#include "spec.h"

/* System includes */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include <getopt.h>

/* Global configuration */
static char *configfile = NULL;
static char *configstring = NULL;
static int verbosity = 1;

/* Child function: startup MoreBalance */
static void launch()
{
	conf c;

	if(configstring)
		c = conf_readstring(configstring);
	else
		c = conf_read(configfile);

	if(c.valid)
	{
		if(verbosity)
		{
			printf(_("Configuration loaded from %s.\n"),
				(configstring ? _("(command line)") : configfile));
		}

		conf_optimize(&c);
		if(verbosity) conf_dump(c);
		if(verbosity) conf_check(c);

		if(verbosity) printf(_("Start the server.\n"));

		server_start(c);
	}
	else
	{
		if(configstring)
			fprintf(stderr, _("Error reading configuration commands.\n"));
		else
			fprintf(stderr, _("Error reading configuration file (%s).\n"),
				configfile);
		exit(-1);
	}
}

/* Available options */
static struct option const options[] =
{
	{"configuration", required_argument, NULL, 'c'},
	{"daemon", no_argument, NULL, 'd'},
	{"help", no_argument, NULL, 'h'},
	{"immediate", required_argument, NULL, 'i'},
	{"spec", no_argument, NULL, 's'},
	{"version", no_argument, NULL, 'v'},
	{NULL, 0, NULL, 0}
};

/* Main function: main safety loop */
int main(int argc, char *argv[])
{
	pid_t pid;
	int status;
	int daemonmode;
	int c, i;
	FILE *f;

	daemonmode = 0;

#ifdef USE_GETTEXT
	bindtextdomain("morebalance", PREFIX "/share/locale");
	textdomain("morebalance");
	setlocale(LC_ALL, "");
#endif

	setbuf(stdout, NULL);

	while((c = getopt_long(argc, argv, "c:dhi:sv", options, NULL)) != -1)
	{
		switch(c)
		{
			case 'c':
				configfile = optarg;
				break;
			case 'd':
				daemonmode = 1;
				verbosity = 0;
				break;
			case 'h':
				printf(_("MoreBalance - a sophisticated load balancing system\n"));
				printf(_("See http://morebalance.coolprojects.org/ for information\n"));
				printf(_("\n"));
				printf(_("Options:\n"));
				printf(_("[-c | --configuration <file>] Use configuration from <file>\n"));
				printf(_("[-i | --immediate <string>  ] Use command line configuration\n"));
				printf(_("[-d | --daemon              ] Run as daemon in the background\n"));
				printf(_("\n"));
				printf(_("[-h | --help                ] Show this help\n"));
				printf(_("[-v | --version             ] Display version information\n"));
				printf(_("[-s | --spec                ] Display build specification\n"));
				exit(0);
				break;
			case 'i':
				configstring = optarg;
				break;
			case 's':
				printf(_("Build specifications\n"));
				printf(_("Encryption support (gnutls):            "));
#ifdef USE_TLS
				printf(_("yes\n"));
#else
				printf(_("no\n"));
#endif
				printf(_("Pattern matching support (pcre):        "));
#ifdef USE_PCRE
				printf(_("yes\n"));
#else
				printf(_("no\n"));
#endif
				printf(_("Internationalization support (gettext): "));
#ifdef USE_GETTEXT
				printf(_("yes\n"));
#else
				printf(_("no\n"));
#endif
				printf(_("Compression support (zlib):             "));
#ifdef USE_ZLIB
				printf(_("yes\n"));
#else
				printf(_("no\n"));
#endif
				exit(0);
				break;
			case 'v':
				printf(_("MoreBalance %s\n"), VERSION);
				exit(0);
				break;
			default:
				fprintf(stderr, _("Unknown option, try --help.\n"));
				exit(-1);
		}
	}

	if(daemonmode)
	{
		fclose(stdout);
#ifdef LOG_FILE
		stdout = fopen(LOG_FILE, "a");
		setbuf(stdout, NULL);
#endif
	}

	for(i = optind; i < argc; i++)
	{
		configstring = argv[i];
	}

	if((!configfile) && (!configstring)) configfile = "morebalance.conf";

	if(verbosity) printf(_(">> Startup.\n"));

	if(daemonmode)
	{
		pid = fork();
		if(pid > 0) exit(0);
		else if(pid < 0)
		{
			fprintf(stderr, _("Couldn't daemonize,\n"));
			exit(-1);
		}
		if(verbosity) printf(_(">> Running as daemon\n"));
	}

	while(1)
	{
		pid = fork();
		switch(pid)
		{
			case -1:
				fprintf(stderr, _("Couldn't fork.\n"));
				exit(-1);
				break;
			case 0:
				launch();
				exit(-1);
				break;
			default:
#ifdef PID_FILE
				f = fopen(PID_FILE, "w");
				if(f)
				{
					fprintf(f, "%i", pid);
					fclose(f);
				}
#endif
				break;
		}

		if(verbosity) printf(_(">> Loop.\n"));

		wait(&status);
		if(WIFSIGNALED(status))
		{
			if(WTERMSIG(status) != 11) break;
		}
		else break;

		fprintf(stderr, _(">> Child crashed. Restarting.\n"));
	}

	if(daemonmode) printf(_(">> Requested exit.\n"));

#ifdef PID_FILE
	unlink(PID_FILE);
#endif

	return 0;
}

